/*******************************************************************************
* File Name: I2C_1_INT.c  
* Version 2.20
*
* Description:
*  This file provides the source code of Interrupt Service Routine (ISR)
*  for I2C component.
*
*   Note:
*
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "I2C_1.h"  


/**********************************
*      System variables
**********************************/

volatile uint8 I2C_1_State;    /* Current state of I2C state machine */

/* Master variables */
#if (I2C_1_MODE & I2C_1_MODE_MASTER)
    volatile uint8 I2C_1_mstrStatus;           /* Master Status byte */
    volatile uint8 I2C_1_mstrControl;          /* Master Control byte */
    
    /* Transmit buffer variables */
    volatile uint8 * I2C_1_mstrRdBufPtr;       /* Pointer to Master Tx/Rx buffer */
    volatile uint8   I2C_1_mstrRdBufSize;      /* Master buffer size */
    volatile uint8   I2C_1_mstrRdBufIndex;     /* Master buffer Index */
    
    /* Receive buffer variables */
    volatile uint8 * I2C_1_mstrWrBufPtr;       /* Pointer to Master Tx/Rx buffer */
    volatile uint8   I2C_1_mstrWrBufSize;      /* Master buffer size  */
    volatile uint8   I2C_1_mstrWrBufIndex;     /* Master buffer Index */

#endif  /* End (I2C_1_MODE & I2C_1_MODE_MASTER) */

/* Slave variables */
#if (I2C_1_MODE & I2C_1_MODE_SLAVE)
    volatile uint8 I2C_1_slStatus;             /* Slave Status byte */

    #if (I2C_1_ADDR_DECODE == I2C_1_SW_DECODE)
        volatile uint8 I2C_1_slAddress;        /* Software address variable */
    #endif  /* End (I2C_1_ADDR_DECODE == I2C_1_SW_DECODE) */
    
    /* Transmit buffer variables */
    volatile uint8 * I2C_1_slRdBufPtr;         /* Pointer to Transmit buffer */
    volatile uint8   I2C_1_slRdBufSize;        /* Slave Transmit buffer size */
    volatile uint8   I2C_1_slRdBufIndex;       /* Slave Transmit buffer Index */
    
    /* Receive buffer variables */
    volatile uint8 * I2C_1_slWrBufPtr;         /* Pointer to Receive buffer */
    volatile uint8   I2C_1_slWrBufSize;        /* Slave Receive buffer size */
    volatile uint8   I2C_1_slWrBufIndex;       /* Slave Receive buffer Index */

#endif  /* End (I2C_1_MODE & I2C_1_MODE_SLAVE) */


/*******************************************************************************
* Function Name: I2C_1_ISR
********************************************************************************
*
* Summary:
*  Handle Interrupt Service Routine.  
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  No
*
*******************************************************************************/
CY_ISR(I2C_1_ISR)
{
    #if (I2C_1_MODE & I2C_1_MODE_SLAVE)
       static uint8  tmp8;    /* Making these static so not wasting time allocating */
    #endif  /* End (I2C_1_MODE & I2C_1_MODE_SLAVE) */
    
    static uint8  tmpCsr;  /* on the stack each time and no one else can see them */
    
    /* Entry from interrupt */
    /* In hardware address compare mode, we can assume we only get interrupted when */
    /* a valid address is recognized. In software address compare mode, we have to  */
    /* check every address after a start condition.                                 */
    
    tmpCsr = I2C_1_CSR_REG;          /* Make temp copy so that we can check */
                                                /* for stop condition after we are done */
    
    #if(I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE)
       
       /* MultiMaster-Slave */
        #if(I2C_1_MODE == I2C_1_MODE_MULTI_MASTER_SLAVE)
            /* Check if START has been generated */
            if((I2C_1_MCSR_REG & I2C_1_MCSR_START_GEN) != 0u)
            {
                /* Clear if was not generated */
                I2C_1_MCSR_REG &= ~I2C_1_MCSR_START_GEN;
            }
        #endif  /* End (I2C_1_MODE == I2C_1_MODE_MULTI_MASTER_SLAVE) */
                
        /* Check for loss of arbitration  */
        if(I2C_1_CHECK_LOST_ARB(tmpCsr))
        {
            /* MultiMaster-Slave */
            #if(I2C_1_MODE == I2C_1_MODE_MULTI_MASTER_SLAVE)    
            /* Check on which state of transaction lost ARBITRAGE:
              Address    - if Address and pass control to Slave
              No Address - release the bus */
            if((tmpCsr & I2C_1_CSR_ADDRESS) == 0u)
            {
                /* Lost ARBITRAGE:
                   On Adress, R/W bit, Data - reset state machine to IDLE
                   On ACK/NACK bit - pass control to SM_MSTR_WAIT_STOP state 
                                     to release the bus */
                if(I2C_1_State != I2C_1_SM_MSTR_WAIT_STOP)
                {
                    /* Clear ByteComplete to clear Enable Slave event */
                    tmpCsr &= ~I2C_1_CSR_BYTE_COMPLETE;
                    
                    /* Clear CSR to release the bus */
                    I2C_1_READY_TO_READ;
                    I2C_1_State = I2C_1_SM_IDLE;  /* Reset State Machine to IDLE */
                }
            }
            else
            {
                /* Let Slave decide the next step, the Stop status will be cleared if set */
                I2C_1_State = I2C_1_SM_IDLE;  /* Reset State Machine to IDLE */
            }
            
            /* MultiMaster */
            #else
                /* Lost ARBITRAGE:
                   On Adress, R/W bit, Data - reset state machine to IDLE
                   On ACK/NACK bit - pass control to SM_MSTR_WAIT_STOP state 
                                     to release the bus */
                if(I2C_1_State != I2C_1_SM_MSTR_WAIT_STOP)
                {
                    /* Clear CSR to release the bus, if MultiMaster */
                    I2C_1_READY_TO_READ;
                    I2C_1_State = I2C_1_SM_IDLE;  /* Reset State Machine to IDLE */
                }
            #endif  /* (I2C_1_MODE == I2C_1_MULTI_MASTER_ENABLE) */
            
            /* Set status error transfer and arbitration lost */
            I2C_1_mstrStatus |= (I2C_1_MSTAT_ERR_ARB_LOST | I2C_1_MSTAT_ERR_XFER);
        }
    #endif  /* End (I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE) */
     
    /* Check for Master operation mode */
    if(I2C_1_State & I2C_1_SM_MASTER)    /*******  Master *******/
    {
        #if (I2C_1_MODE & I2C_1_MODE_MASTER)
        /* Enter Master state machine */
        
            if(I2C_1_CHECK_BYTE_COMPLETE(tmpCsr))
            {
                #if(I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE)
                    tmpCsr &= ~I2C_1_CSR_STOP_STATUS;  /* Clear STOP bit */
                #endif  /* End (I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE) */
                
                switch(I2C_1_State)
                {
                    case I2C_1_SM_MSTR_WR_ADDR:    /* After address is sent, WRITE some data */
                    case I2C_1_SM_MSTR_RD_ADDR:    /* After address is sent, READ some data */
                    
                        /* Check for Slave address ACK */
                        if(I2C_1_CHECK_ADDR_ACK(tmpCsr))  /* Check ACK */
                        {
                            /* Setup for transmit or receive of data */
                            if(I2C_1_State == I2C_1_SM_MSTR_WR_ADDR)   /* TRANSMIT data */
                            {
                                if(I2C_1_mstrWrBufSize > 0)    /* Check if at least one byte is transfered */
                                {
                                    /* Load first data byte */
                                    I2C_1_DATA_REG = I2C_1_mstrWrBufPtr[0u];
                                    I2C_1_TRANSMIT_DATA;            /* Transmit data */
                                    I2C_1_mstrWrBufIndex = 1u;      /* Set index to 2nd location */
                                    
                                    /* Set transmit state until done */
                                    I2C_1_State = I2C_1_SM_MSTR_WR_DATA; 
                                }
                                else   /* No data to tranfer */
                                {
                                    /* Handles 0 bytes transfer */
                                    #if (I2C_1_PSOC3_ES2 || I2C_1_PSOC5_ES1)
                                        I2C_1_GENERATE_STOP;     /* Generate STOP */
                                        
                                        /* Set WRITE complete */
                                        I2C_1_mstrStatus |= I2C_1_MSTAT_WR_CMPLT;
                                        
                                        /* Reset State Machine to IDLE */
                                        I2C_1_State  = I2C_1_SM_IDLE; 
                                        
                                    #else  /* The PSoC3 ES3 only handles this well */
                                        if(I2C_1_CHECK_NO_STOP(I2C_1_mstrControl))
                                        {
                                            /* Reset State Machine to HALT, expect RESTART */
                                            I2C_1_State  = I2C_1_SM_MSTR_HALT;
                                            
                                            /* Set WRITE complete and Master HALTED */
                                            I2C_1_mstrStatus |= (I2C_1_MSTAT_XFER_HALT | 
                                                                            I2C_1_MSTAT_WR_CMPLT); 
                                            
                                            CyIntClearPending(I2C_1_ISR_NUMBER);
                                            I2C_1_DisableInt();
                                        }
                                        else  /* Do normal STOP */
                                        {
                                            I2C_1_GENERATE_STOP;     /* Generate STOP */
                                             
                                            /* Set WRITE complete */
                                            I2C_1_mstrStatus |= I2C_1_MSTAT_WR_CMPLT;
                                        
                                             /* Reset State Machine to IDLE */
                                            I2C_1_State = I2C_1_SM_IDLE;
                                        }
                                        
                                    #endif  /* End (I2C_1_PSOC3_ES2 || I2C_1_PSOC5_ES1) */
                                }
                            }
                            else  /* Master Receive data */
                            {                                
                                I2C_1_READY_TO_READ;     /* Ready to READ data */
                                
                                /* Set state machine to READ data */
                                I2C_1_State  = I2C_1_SM_MSTR_RD_DATA;
                            }
                        }
                        /* Check for Slave address NAK */
                        else if(I2C_1_CHECK_ADDR_NAK(tmpCsr))  /* Check NACK */
                        {
                            if(I2C_1_CHECK_NO_STOP(I2C_1_mstrControl))
                            {
                                /* Reset State Machine to HALT, expect RESTART */
                                I2C_1_State  = I2C_1_SM_MSTR_HALT;
                                
                                /* Set Address NAK Error and Master HALTED */
                                I2C_1_mstrStatus |= I2C_1_MSTAT_ERR_ADDR_NAK |
                                                               I2C_1_MSTAT_XFER_HALT;
                                
                                CyIntClearPending(I2C_1_ISR_NUMBER);
                                I2C_1_DisableInt();
                            }
                            else  /* Do normal Stop */
                            {
                                I2C_1_GENERATE_STOP;     /* Generate STOP */
                                I2C_1_State = I2C_1_SM_IDLE;  /* Reset State Machine to idle */
                                
                                /* Set Address NAK Error */
                                I2C_1_mstrStatus |= I2C_1_MSTAT_ERR_ADDR_NAK; 
                            }
                        }
                        /* Bogus */
                        else
                        {
                            CyIntClearPending(I2C_1_ISR_NUMBER);
                            I2C_1_DisableInt();
                        }
                        break;
                        
                    case I2C_1_SM_MSTR_WR_DATA:    /* Write data to slave */
                        
                        if(I2C_1_CHECK_DATA_ACK(tmpCsr))       /* Check ACK */
                        {
                            if(I2C_1_mstrWrBufIndex  < I2C_1_mstrWrBufSize) /* Check if end buf */
                            {
                                 /* Load first data byte  */
                                I2C_1_DATA_REG = I2C_1_mstrWrBufPtr[I2C_1_mstrWrBufIndex];
                                I2C_1_TRANSMIT_DATA;     /* Transmit */
                                
                                I2C_1_mstrWrBufIndex++;  /* Advance to data location */
                            }
                            else   /* Last byte was transmitted, send STOP */
                            {
                                if(I2C_1_CHECK_NO_STOP(I2C_1_mstrControl))
                                {
                                    /* Reset State Machine to HALT, expect RESTART */
                                    I2C_1_State  = I2C_1_SM_MSTR_HALT;
                                    
                                    /* Set WRITE complete and Master HALTED */
                                    I2C_1_mstrStatus |= (I2C_1_MSTAT_WR_CMPLT |
                                                                    I2C_1_MSTAT_XFER_HALT);
                                    
                                    CyIntClearPending(I2C_1_ISR_NUMBER);
                                    I2C_1_DisableInt();
                                }
                                else  /* Do normal Stop */
                                {
                                    I2C_1_Workaround();      /* Workaround for CDT 78083 */
                                    I2C_1_GENERATE_STOP;     /* Generate STOP */
                                    I2C_1_State = I2C_1_SM_IDLE; /* Reset State Machine to idle */
                                    
                                    /* Set WRITE complete */
                                    I2C_1_mstrStatus |= I2C_1_MSTAT_WR_CMPLT;
                                }
                            }
                        }
                        else /* If last byte NAKed, stop transmit and send STOP */
                        {
                            if(I2C_1_CHECK_NO_STOP(I2C_1_mstrControl)) /* Check STOP generation */
                            {
                                /* Reset State Machine to HALT, expect RESTART */
                                I2C_1_State  = I2C_1_SM_MSTR_HALT;
                                
                                /* Set WRITE complete, SHORT transfer and Master HALTED */
                                I2C_1_mstrStatus |= (I2C_1_MSTAT_WR_CMPLT |
                                                                I2C_1_MSTAT_ERR_SHORT_XFER |
                                                                I2C_1_MSTAT_XFER_HALT);
                                
                                CyIntClearPending(I2C_1_ISR_NUMBER);  
                                I2C_1_DisableInt();
                            }
                            else  /* Do normal Stop */
                            {
                                I2C_1_GENERATE_STOP;         /* Generate STOP */
                                I2C_1_State = I2C_1_SM_IDLE;    /* Reset State Machine to IDLE */
                                
                                /* Set WRITE complete and SHORT transfer */
                                I2C_1_mstrStatus |= (I2C_1_MSTAT_WR_CMPLT | 
                                                                I2C_1_MSTAT_ERR_SHORT_XFER);
                            }
                        }
                        break;
                        
                    case I2C_1_SM_MSTR_RD_DATA:    /* Data received */
                        
                        I2C_1_mstrRdBufPtr[I2C_1_mstrRdBufIndex] = I2C_1_DATA_REG;
                        I2C_1_mstrRdBufIndex++;      /* Inc pointer */
                        if(I2C_1_mstrRdBufIndex < I2C_1_mstrRdBufSize)/* Check if end of buffer */
                        {
                            I2C_1_ACK_AND_RECEIVE;       /* ACK and receive */
                        }
                        else   /* End of data, generate a STOP */
                        {
                            if(I2C_1_CHECK_NO_STOP(I2C_1_mstrControl)) /* Check STOP generation */
                            {
                                /* Reset State Machine to HALT, expect RESTART */
                                I2C_1_State = I2C_1_SM_MSTR_HALT;    
                                
                                /* Set READ complete and Master HALTED */
                                I2C_1_mstrStatus |= (I2C_1_MSTAT_RD_CMPLT |
                                                                I2C_1_MSTAT_XFER_HALT );
                            }
                            else   /* Do normal Stop */
                            {
                                #if(I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE)
                                    I2C_1_NAK_AND_RECEIVE;           /* NACK and TRY to generate STOP */
                                    I2C_1_ENABLE_INT_ON_STOP;        /* Enable interrupt on STOP */
                                     /* Wait for STOP or LOST ARBITRAGE */
                                    I2C_1_State = I2C_1_SM_MSTR_WAIT_STOP;
                                    
                                #else
                                    I2C_1_NAK_AND_RECEIVE;                   /* NACK and and generate STOP */
                                    I2C_1_State = I2C_1_SM_IDLE;  /* Set state to IDLE */
                                    
                                    /* Set READ complete */
                                    I2C_1_mstrStatus |= I2C_1_MSTAT_RD_CMPLT;
                                    
                                #endif  /* End (I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE) */
                            }
                        }
                        break;
                        
                    /* Only wait for STOP if MultiMaster is used */
                    #if(I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE)
                        case I2C_1_SM_MSTR_WAIT_STOP:
                            I2C_1_READY_TO_READ;         /* The ARBITRAGE has been lost, release the bus */
                            I2C_1_DISABLE_INT_ON_STOP;                        /* Disable interrupt on STOP */
                            I2C_1_State = I2C_1_SM_IDLE;           /* Set state to IDLE */
                            /* Set status to complete READ */
                            I2C_1_mstrStatus |= I2C_1_MSTAT_RD_CMPLT;
                            break;
                    #endif  /* End (I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE) */
                                            
                    /* This case used for NO STOP condition, ready for a restart */
                    case I2C_1_SM_MSTR_HALT: /* Do one transfer and halt, used for polling or single stepping */
                        I2C_1_mstrStatus |= I2C_1_MSTAT_WR_CMPLT;
                        I2C_1_DisableInt();
                        break;
                        
                    default:
                        /* Invalid state, reset state machine  to a known state */
                        I2C_1_State = I2C_1_SM_IDLE;
                        
                        /* Set transfer complete with error */
                        I2C_1_mstrStatus |= I2C_1_MSTAT_ERR_XFER;
                        break;
                }
            }
            
            #if(I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE)
                /* Check if STOP was detected */
                if((tmpCsr & I2C_1_CSR_STOP_STATUS) != 0u)
                {
                    /* The READ transaction only gets here on STOP */
                    I2C_1_DISABLE_INT_ON_STOP;
                    I2C_1_State = I2C_1_SM_IDLE;  /* Set state to IDLE */
                    
                    /* Set status to complete READ */
                    I2C_1_mstrStatus |= I2C_1_MSTAT_RD_CMPLT;
                }
            #endif  /* End (I2C_1_MODE & I2C_1_MODE_MULTI_MASTER_ENABLE) */
            
        #endif  /* End (I2C_1_MODE & I2C_1_MODE_MASTER) */
    }
    else    /******** Slave Mode ********/
    {
        #if (I2C_1_MODE & I2C_1_MODE_SLAVE)
            /* Check to see if a Start/Address is detected */
            if((tmpCsr & I2C_1_CSR_ADDRESS) != 0u)
            {
                /* Clears CSR STOP_STATUS bit. This bit sets by ANY of Stop condition detection on the bus */
                tmpCsr &= ~I2C_1_CSR_STOP_STATUS;  /* Clear Stop bit */
                
                /* This is a Start or ReStart, So Reset the state machine, Check for a Read/Write condition */
                    
                /* Check for software address detection */
                #if (I2C_1_ADDR_DECODE == I2C_1_SW_DECODE)  
                    /******************* Software address detection ************************/
                    tmp8 = ((I2C_1_DATA_REG >> 1u) & I2C_1_SLAVE_ADDR_MASK);
                    if(tmp8 == I2C_1_slAddress)   /* Check for address match  */
                    {
                        /* Check for read or write command */
                        if((I2C_1_DATA_REG & I2C_1_READ_FLAG) != 0u) 
                        {
                            /*******************************************/
                            /*  Place code to prepare read buffer here */
                            /*******************************************/
                            /* `#START SW_PREPARE_READ_BUF` */

                            /* `#END`  */
                            
                            /* Prepare next opeation to read, get data and place in data register */
                            if(I2C_1_slRdBufIndex < I2C_1_slRdBufSize)
                            {
                                /* Load first data byte */
                                I2C_1_DATA_REG = I2C_1_slRdBufPtr[I2C_1_slRdBufIndex];
                                I2C_1_ACK_AND_TRANSMIT;  /* ACK and transmit */
                                I2C_1_slRdBufIndex++;    /* Advance to data location */
                                
                                /* Set Read activity */
                                I2C_1_slStatus |= I2C_1_SSTAT_RD_BUSY; 
                            }
                            else    /* Data overflow */
                            {
                                I2C_1_DATA_REG = 0xFFu;    /* Out of range, send 0xFF  */
                                I2C_1_ACK_AND_TRANSMIT;    /* ACK and transmit */
                                
                                /* Set Read activity */
                                I2C_1_slStatus  |= (I2C_1_SSTAT_RD_BUSY | 
                                                               I2C_1_SSTAT_RD_ERR_OVFL); 
                            }
                            
                            I2C_1_State = I2C_1_SM_SL_RD_DATA; /* Prepare for Read transaction */
                        }
                        else  /* Start of a Write transaction, ready to write of the first byte */
                        {
                            /* Prepare to write the first byte */
                            I2C_1_ACK_AND_RECEIVE;
                            I2C_1_State = I2C_1_SM_SL_WR_DATA; /* Prepare for Write transaction */
                            
                            /* Set Write activity */
                            I2C_1_slStatus |= I2C_1_SSTAT_WR_BUSY;
                            I2C_1_ENABLE_INT_ON_STOP;    /* Enable interrupt on Stop */
                        }
                    }
                    /**********************************************/
                    /* Place compare for additional address here  */
                    /**********************************************/
                    /* `#START ADDR_COMPARE`  */

                    /* `#END`  */
                    
                    else   /* No address match */
                    {
                        I2C_1_NAK_AND_RECEIVE;   /* NAK address */
                    }
                    
                #else  /* Hardware address detection */
                    /* Check for read or write command */
                    if((I2C_1_DATA_REG & I2C_1_READ_FLAG) != 0u)
                    {
                        /*******************************************/
                        /*  Place code to prepare read buffer here */
                        /*******************************************/
                        /* `#START SW_PREPARE_READ_BUF` */

                        /* `#END`  */
                         
                        /* Prepare next opeation to read, get data and place in data register */
                        if(I2C_1_slRdBufIndex < I2C_1_slRdBufSize)
                        {
                            /* Load first data byte */
                            I2C_1_DATA_REG = I2C_1_slRdBufPtr[I2C_1_slRdBufIndex];
                            I2C_1_ACK_AND_TRANSMIT;                            /* ACK and transmit */
                            I2C_1_slRdBufIndex++;                              /* Advance to data location */
                            
                            /* Set Read activity */
                            I2C_1_slStatus  |= I2C_1_SSTAT_RD_BUSY;
                        }
                        else    /* Data overflow */
                        {
                            I2C_1_DATA_REG = 0xFFu;    /* Out of range, send 0xFF  */
                            I2C_1_ACK_AND_TRANSMIT;    /* ACK and transmit */
                            
                            /* Set Read activity */
                            I2C_1_slStatus  |= (I2C_1_SSTAT_RD_BUSY |
                                                           I2C_1_SSTAT_RD_ERR_OVFL);
                        }
                        
                        I2C_1_State = I2C_1_SM_SL_RD_DATA;    /* Prepare for Read transaction */
                    }
                    else  /* Start of a Write transaction, ready to write of the first byte */
                    {
                        /* Prepare to write the first byte */
                        I2C_1_ACK_AND_RECEIVE;       /* ACK and ready to receive addr */
                        I2C_1_State = I2C_1_SM_SL_WR_DATA;    /* Prepare for write transaction */
                        
                        /* Set Write activity */
                        I2C_1_slStatus |= I2C_1_SSTAT_WR_BUSY;
                        I2C_1_ENABLE_INT_ON_STOP;    /* Enable interrupt on Stop */
                    }
                    
                #endif  /* End (I2C_1_ADDR_DECODE == I2C_1_SW_DECODE) */
            }
            else if(I2C_1_CHECK_BYTE_COMPLETE(tmpCsr))    /* Check for data transfer */
            {
                if (I2C_1_State == I2C_1_SM_SL_WR_DATA)    /* Data write from Master to Slave */
                {
                    if(I2C_1_slWrBufIndex < I2C_1_slWrBufSize)       /* Check for valid range */
                    {
                        tmp8 = I2C_1_DATA_REG;                        /* Get data, to ACK quickly */
                        I2C_1_ACK_AND_RECEIVE;                        /* ACK and ready to receive */
                        I2C_1_slWrBufPtr[I2C_1_slWrBufIndex] = tmp8; /* Write data to array */
                        I2C_1_slWrBufIndex++;                        /* Inc pointer */
                    }
                    else
                    {
                        I2C_1_NAK_AND_RECEIVE;       /* NAK cause beyond write area */
                        
                        /* Set overflow, write completes on Stop */
                        I2C_1_slStatus |= I2C_1_SSTAT_WR_ERR_OVFL;
                    }
                }
                else if (I2C_1_State == I2C_1_SM_SL_RD_DATA) /* Data Read from Slave to Master */
                {
                    if(I2C_1_CHECK_DATA_ACK(tmpCsr))
                    {
                        if(I2C_1_slRdBufIndex < I2C_1_slRdBufSize) 
                        {
                             /* Get data from array */
                            I2C_1_DATA_REG = I2C_1_slRdBufPtr[I2C_1_slRdBufIndex];
                            I2C_1_TRANSMIT_DATA;         /* Send Data */
                            I2C_1_slRdBufIndex++;        /* Inc pointer */
                        }
                        else   /* Over flow */
                        {
                            I2C_1_DATA_REG = 0xFFu;          /* Send 0xFF at the end of the buffer */
                            I2C_1_TRANSMIT_DATA;             /* Send Data */
                            
                            /* Set overflow */
                            I2C_1_slStatus |= I2C_1_SSTAT_RD_ERR_OVFL;
                        }
                    }
                    else  /* Last byte NAKed, done */
                    {
                        I2C_1_DATA_REG = 0xFFu;      /* End of read transaction */
                        
                        I2C_1_NAK_AND_TRANSMIT;      /* Clear transmit bit at the end of read transaction */
                         /* Clear Busy Flag */
                        I2C_1_slStatus &= ~I2C_1_SSTAT_RD_BUSY;
                        /* Set Read complete Flag */
                        I2C_1_slStatus |= I2C_1_SSTAT_RD_CMPT;
                        
                        I2C_1_State = I2C_1_SM_IDLE;      /* Return to IDLE state */
                    }
                }
                else  /* This is an invalid state and should not occur */
                {
                    /* Invalid state, Reset */
                    I2C_1_State = I2C_1_SM_IDLE;
                    I2C_1_NAK_AND_RECEIVE;
                }   /* End Transfer mode */
            }  
            else if ((tmpCsr & I2C_1_CSR_BUS_ERROR) != 0u)    /* Quick check for Error */
            {
                if((I2C_1_CSR_REG & I2C_1_CSR_BUS_ERROR) != 0u)
                {
                    /* May want to reset bus here CHECK */
                }
            }  /* End if without else */
            
            if((tmpCsr & I2C_1_CSR_STOP_STATUS) != 0u)    /* Check if STOP was detected */
            {
                /* The Write transaction only IE on Stop, so Read never gets here */
                /* The WR_BUSY flag will be cleared at the end of "Write-ReStart-Read-Stop" transaction */
                /* Clear Busy Flag */
                I2C_1_slStatus &= ~I2C_1_SSTAT_WR_BUSY;
                /* Set complete Flag */
                I2C_1_slStatus |= I2C_1_SSTAT_WR_CMPT;
                
                I2C_1_DISABLE_INT_ON_STOP;               /* Disable interrupt on STOP */
                I2C_1_State = I2C_1_SM_IDLE;  /* Return to IDLE */
            }
        
        #endif  /* End (I2C_1_MODE & I2C_1_MODE_SLAVE) */
    }
    
    #if (I2C_1_PSOC3_ES2 && (I2C_1_I2C_IRQ__ES2_PATCH))
        I2C_1_ISR_PATCH();
    #endif  /* End (I2C_1_PSOC3_ES2 && (I2C_1_I2C_IRQ__ES2_PATCH)) */
}


/* [] END OF FILE */
